//`include "std_dff.sv"

module i2s2pcm (
	input logic bclk, lrclk, data, rst,
	output logic doutleft, doutright, le, bclkout
);

	logic 	[15:0]	leftinbuffer, rightinbuffer;
	logic	[15:0]	leftoutbuffer1, rightoutbuffer1, leftoutbuffer1_next, rightoutbuffer1_next;
	logic	[15:0]	leftoutbuffer2, rightoutbuffer2, leftoutbuffer2_next, rightoutbuffer2_next;
	logic	[4:0]	leftcount, leftcount_next, rightcount, rightcount_next;
	logic	[3:0]	outcount, outcount_next;
	logic	bclkdividewire, doutleft_next, doutright_next, leftinfull, rightinfull, outready, le_int;
	
	//bclk Divide by 4 logic
	std_dff clkdivide1(.clk(bclk), .rst(rst), .d(~bclkdividewire), .q(bclkdividewire));
	std_dff clkdivide2(.clk(bclkdividewire), .rst(rst), .d(~bclkout), .q(bclkout));
	
	always_ff @ (posedge bclk, posedge rst) begin
		if (rst) begin
			leftcount <= 31;
			rightcount <= 31;
			leftinbuffer <= 0;
			rightinbuffer <= 0;
			leftoutbuffer1 <= 0;
			rightoutbuffer1 <= 0;
		end
		else begin
			leftcount <= leftcount_next;
			rightcount <= rightcount_next;
			if (lrclk == 1) rightinbuffer[rightcount] <= data;
			else leftinbuffer[leftcount] <= data;
			
			if (rightinfull == 1) rightoutbuffer1 <= rightoutbuffer1_next;
			if (leftinfull == 1) leftoutbuffer1 <= leftoutbuffer1_next;
		end
	end
	
	always_ff @ (posedge bclkout, posedge rst) begin	
		if (rst) begin
			outcount <= 0;
			doutleft <= 0;
			doutright <= 0;
			leftoutbuffer2 <= 0;
			rightoutbuffer2 <= 0;
			le_int <= 0;
		end
		else begin
			doutleft <= doutleft_next;
			doutright <= doutright_next;
			outcount <= outcount_next;
			if (outcount == 15) le_int <= 1;
			else le_int <= 0;
			if (outready) begin
				leftoutbuffer2 <= leftoutbuffer2_next;
				rightoutbuffer2 <= rightoutbuffer2_next;
			end
		end
	end
	
	always_comb begin
		if (lrclk == 1) begin
			if (rightcount == 18) rightinfull = 1;
			else rightinfull = 0;			
			rightcount_next = rightcount + 1'b1;
			leftcount_next = 31;
			leftinfull = 0;
		end
		else begin
			if (leftcount == 18) leftinfull = 1;
			else leftinfull = 0;			
			leftcount_next = leftcount + 1'b1;
			rightcount_next = 31;
			rightinfull = 0;
		end
		rightoutbuffer1_next = rightinbuffer[15:0];
		leftoutbuffer1_next = leftinbuffer[15:0];
		rightoutbuffer2_next = rightoutbuffer1;
		leftoutbuffer2_next = leftoutbuffer1;
		
		outcount_next = outcount + 1'b1;
		if (outcount == 15) outready = 1;
		else outready = 0;
		
		// Invert MSB
		if (outcount == 0) begin
			doutleft_next = ~leftoutbuffer2[outcount];
			doutright_next = ~rightoutbuffer2[outcount];
		end
		else begin
			doutleft_next = leftoutbuffer2[outcount];
			doutright_next = rightoutbuffer2[outcount];
		end
		//rstn = !GSR;
		le = !bclkout&le_int;
	end

endmodule